from typing import Any

import fitbit

from fitbit_downloader.authorize import set_authorization, initial_client_authorization
from fitbit_downloader.config import Config, OAuthConfig


def get_client(config: Config) -> fitbit.Fitbit:
    if config.oauth_config is None:
        initial_client_authorization(config)

    def on_refresh_token(raw_token_data: dict[str, Any]):
        set_authorization(config, raw_token_data)

    oa: OAuthConfig = config.oauth_config  # type: ignore
    return fitbit.Fitbit(
        config.client_id,
        config.client_secret,
        oauth2=True,
        access_token=oa.access_token,
        refresh_token=oa.refresh_token,
        expires_at=oa.expires_at,
        refresh_cb=on_refresh_token,
    )



#!/usr/bin/env python
import cherrypy
import os
import sys
import threading
import traceback
import webbrowser

from urllib.parse import urlparse
from base64 import b64encode
from fitbit.api import Fitbit
from oauthlib.oauth2.rfc6749.errors import MismatchingStateError, MissingTokenError


class OAuth2Server:
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        redirect_uri: str = "http://127.0.0.1:8080/",
    ):
        """Initialize the FitbitOauth2Client"""
        self.success_html = """
            <h1>You are now authorized to access the Fitbit API!</h1>
            <br/><h3>You can close this window</h3>"""
        self.failure_html = """
            <h1>ERROR: %s</h1><br/><h3>You can close this window</h3>%s"""

        self.fitbit = Fitbit(
            client_id,
            client_secret,
            redirect_uri=redirect_uri,
            timeout=10,
        )

        self.redirect_uri = redirect_uri

    def browser_authorize(self):
        """
        Open a browser to the authorization url and spool up a CherryPy
        server to accept the response
        """
        url, _ = self.fitbit.client.authorize_token_url()
        # Open the web browser in a new thread for command-line browser support
        threading.Timer(1, webbrowser.open, args=(url,)).start()

        # Same with redirect_uri hostname and port.
        urlparams = urlparse(self.redirect_uri)
        cherrypy.config.update(
            {
                "server.socket_host": urlparams.hostname,
                "server.socket_port": urlparams.port,
            }
        )

        cherrypy.quickstart(self)

    @cherrypy.expose
    def index(self, state, code=None, error=None):
        """
        Receive a Fitbit response containing a verification code. Use the code
        to fetch the access_token.
        """
        error = None
        if code:
            try:
                self.fitbit.client.fetch_access_token(code)
            except MissingTokenError:
                error = self._fmt_failure(
                    "Missing access token parameter.</br>Please check that "
                    "you are using the correct client_secret"
                )
            except MismatchingStateError:
                error = self._fmt_failure("CSRF Warning! Mismatching state")
        else:
            error = self._fmt_failure("Unknown error while authenticating")
        # Use a thread to shutdown cherrypy so we can return HTML first
        self._shutdown_cherrypy()
        return error if error else self.success_html

    def _fmt_failure(self, message):
        tb = traceback.format_tb(sys.exc_info()[2])
        tb_html = "<pre>%s</pre>" % ("\n".join(tb)) if tb else ""
        return self.failure_html % (message, tb_html)

    def _shutdown_cherrypy(self):
        """Shutdown cherrypy in one second, if it's running"""
        if cherrypy.engine.state == cherrypy.engine.states.STARTED:
            threading.Timer(1, cherrypy.engine.exit).start()


if __name__ == "__main__":

    if not (len(sys.argv) == 3):
        print("Arguments: client_id and client_secret")
        sys.exit(1)

    server = OAuth2Server(*sys.argv[1:])
    server.browser_authorize()

    profile = server.fitbit.user_profile_get()
    print(
        "You are authorized to access data for the user: {}".format(
            profile["user"]["fullName"]
        )
    )

    print("TOKEN\n=====\n")
    for key, value in server.fitbit.client.session.token.items():
        print("{} = {}".format(key, value))



# generated by datamodel-codegen:
#   filename:  elevation.json
#   timestamp: 2021-10-31T18:17:22+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Field


class ActivitiesElevationItem(BaseModel):
    dateTime: str
    value: str


class DatasetItem(BaseModel):
    time: str
    value: int


class ActivitiesElevationIntraday(BaseModel):
    dataset: List[DatasetItem]
    datasetInterval: int
    datasetType: str


class ElevationResponse(BaseModel):
    activities_elevation: List[ActivitiesElevationItem] = Field(
        ..., alias='activities-elevation'
    )
    activities_elevation_intraday: ActivitiesElevationIntraday = Field(
        ..., alias='activities-elevation-intraday'
    )



# generated by datamodel-codegen:
#   filename:  heart.json
#   timestamp: 2021-10-31T18:17:22+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Field


class HeartRateZone(BaseModel):
    caloriesOut: float
    max: int
    min: int
    minutes: int
    name: str


class Value(BaseModel):
    customHeartRateZones: List
    heartRateZones: List[HeartRateZone]
    restingHeartRate: int


class ActivitiesHeartItem(BaseModel):
    dateTime: str
    value: Value


class DatasetItem(BaseModel):
    time: str
    value: int


class ActivitiesHeartIntraday(BaseModel):
    dataset: List[DatasetItem]
    datasetInterval: int
    datasetType: str


class HeartResponse(BaseModel):
    activities_heart: List[ActivitiesHeartItem] = Field(..., alias='activities-heart')
    activities_heart_intraday: ActivitiesHeartIntraday = Field(
        ..., alias='activities-heart-intraday'
    )



# generated by datamodel-codegen:
#   filename:  distance.json
#   timestamp: 2021-10-31T18:17:23+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Field


class ActivitiesDistanceItem(BaseModel):
    dateTime: str
    value: str


class DatasetItem(BaseModel):
    time: str
    value: float


class ActivitiesDistanceIntraday(BaseModel):
    dataset: List[DatasetItem]
    datasetInterval: int
    datasetType: str


class DistanceResponse(BaseModel):
    activities_distance: List[ActivitiesDistanceItem] = Field(
        ..., alias='activities-distance'
    )
    activities_distance_intraday: ActivitiesDistanceIntraday = Field(
        ..., alias='activities-distance-intraday'
    )



import os
from pathlib import Path
from typing import Generator

from fitbit_downloader.ext_subprocess import run_then_show_output
from tests.config import RESPONSES_FOLDER

MODELS_FOLDER = Path(__file__).parent / "models"


def generate_models():
    for path in _get_json_paths():
        model_name = _create_model_name(path)
        print(f"Generating model {model_name} from {path}")
        out_path = MODELS_FOLDER / f"{model_name.lower()}.py"
        _generate_model(path, out_path, model_name)


def _create_model_name(path: Path) -> str:
    base_name = path.with_suffix("").name
    parts = base_name.split("-")
    combined = "".join(part.title() for part in parts)
    return f"{combined}Response"


def _get_json_paths() -> Generator[Path, None, None]:
    for path, folders, files in os.walk(RESPONSES_FOLDER):
        for file in files:
            if file.endswith(".json"):
                full_path = Path(path) / file
                yield full_path


def _generate_model(in_path: Path, out_path: Path, name: str):
    command = f"datamodel-codegen --input {in_path.absolute()} --input-file-type json --output {out_path} --class-name {name}"
    run_then_show_output(command)


def main():
    generate_models()


if __name__ == "__main__":
    main()



# generated by datamodel-codegen:
#   filename:  refresh-token.json
#   timestamp: 2021-10-31T18:17:20+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel


class RefreshTokenResponse(BaseModel):
    access_token: str
    expires_in: int
    refresh_token: str
    scope: List[str]
    token_type: str
    user_id: str
    expires_at: float



from pathlib import Path

TESTS_FOLDER = Path(__file__).parent
INPUT_DATA_FOLDER = TESTS_FOLDER / "input_data"
RESPONSES_FOLDER = INPUT_DATA_FOLDER / "responses"
INTRADAY_RESPONSES_FOLDER = RESPONSES_FOLDER / "intraday"


# generated by datamodel-codegen:
#   filename:  steps.json
#   timestamp: 2021-10-31T18:17:22+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Field


class ActivitiesStep(BaseModel):
    dateTime: str
    value: str


class DatasetItem(BaseModel):
    time: str
    value: int


class ActivitiesStepsIntraday(BaseModel):
    dataset: List[DatasetItem]
    datasetInterval: int
    datasetType: str


class StepsResponse(BaseModel):
    activities_steps: List[ActivitiesStep] = Field(..., alias='activities-steps')
    activities_steps_intraday: ActivitiesStepsIntraday = Field(
        ..., alias='activities-steps-intraday'
    )



import shlex
import subprocess


def _stream_process(process):
    go = process.poll() is None
    for line in process.stdout:
        print(line.decode("utf8"), end="")
    return go


def run_and_stream_output(command: str):
    process = subprocess.Popen(
        shlex.split(command),
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    )
    while _stream_process(process):
        pass


def run_then_show_output(command: str):
    output = subprocess.run(
        shlex.split(command),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        check=True,
    )
    print(output.stdout.decode("utf8"))



import datetime
import logging
import sys
from pathlib import Path
from typing import Final

LOG_FOLDER: Final[Path] = Path(__file__).parent / "Logs"

if not LOG_FOLDER.exists():
    LOG_FOLDER.mkdir(parents=True)


def _get_log_path(folder: Path = LOG_FOLDER) -> Path:
    current_time = datetime.datetime.now().replace(microsecond=0)
    current_time_str = str(current_time).replace(":", "-")
    return folder / f"{current_time_str}.log"


logger = logging.getLogger("fitbit-downloader")
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s %(levelname)s: %(message)s")

ch = logging.StreamHandler(stream=sys.stdout)
ch.setLevel(logging.DEBUG)
ch.setFormatter(formatter)
logger.addHandler(ch)

fh = logging.FileHandler(_get_log_path())
fh.setLevel(logging.INFO)
fh.setFormatter(formatter)
logger.addHandler(fh)



import datetime
from pathlib import Path
from typing import Optional, Type, Any

import fitbit
from pydantic import BaseModel

from fitbit_downloader.client import get_client
from fitbit_downloader.config import Config, Dataset
from fitbit_downloader.logger import logger
from fitbit_downloader.models.activityresponse import ActivityResponse
from fitbit_downloader.models.distanceresponse import DistanceResponse
from fitbit_downloader.models.elevationresponse import ElevationResponse
from fitbit_downloader.models.floorsresponse import FloorsResponse
from fitbit_downloader.models.heartresponse import HeartResponse
from fitbit_downloader.models.sleepresponse import SleepResponse
from fitbit_downloader.models.stepsresponse import StepsResponse


def download_data(config: Config, custom_date: Optional[datetime.date] = None):
    date = custom_date or _yesterday()
    logger.info(f"Downloading data for {date}")
    client = get_client(config)
    if not config.download.out_folder.exists():
        config.download.out_folder.mkdir()
    for dataset in config.download.datasets:
        logger.info(f"Downloading data for {dataset.value}")
        if dataset in (
            Dataset.STEPS,
            Dataset.HEART,
            Dataset.ELEVATION,
            Dataset.DISTANCE,
            Dataset.FLOORS,
        ):
            _download_intraday_data(dataset, client, config, date)
        elif dataset == Dataset.SLEEP:
            _download_sleep_data(dataset, client, config, date)
        elif dataset == Dataset.ACTIVITIES:
            _download_activity_data(dataset, client, config, date)


def _download_intraday_data(
    dataset: Dataset, client: fitbit.Fitbit, config: Config, date: datetime.date
):
    activity_str = f"activities/{dataset.value}"
    data = client.intraday_time_series(activity_str, base_date=date)
    response_cls = _get_intraday_response_class(dataset)
    out_path = _get_out_path(dataset, config, date)
    _save(out_path, data, response_cls)


def _download_sleep_data(
    dataset: Dataset, client: fitbit.Fitbit, config: Config, date: datetime.date
):
    data = client.get_sleep(date)
    out_path = _get_out_path(dataset, config, date)
    _save(out_path, data, SleepResponse)


def _download_activity_data(
    dataset: Dataset, client: fitbit.Fitbit, config: Config, date: datetime.date
):
    data = client.activities(date=date)  # type: ignore
    out_path = _get_out_path(dataset, config, date)
    _save(out_path, data, ActivityResponse)


def _save(out_path: Path, data: dict[str, Any], response_cls: Type[BaseModel]):
    model = response_cls(**data)
    out_path.write_text(model.json(indent=2))


def _get_out_path(dataset: Dataset, config: Config, date: datetime.date) -> Path:
    out_folder = config.download.out_folder / dataset.value
    if not out_folder.exists():
        out_folder.mkdir()
    return out_folder / (date.strftime("%Y-%m-%d") + ".json")


def _get_intraday_response_class(dataset: Dataset) -> Type[BaseModel]:
    if dataset == Dataset.STEPS:
        return StepsResponse
    if dataset == Dataset.HEART:
        return HeartResponse
    if dataset == Dataset.ELEVATION:
        return ElevationResponse
    if dataset == Dataset.DISTANCE:
        return DistanceResponse
    if dataset == Dataset.FLOORS:
        return FloorsResponse
    raise NotImplementedError(f"need to add handling for intraday {dataset.value}")


def _yesterday() -> datetime.date:
    return datetime.date.today() - datetime.timedelta(days=1)



from fitbit_downloader.config import Config
from fitbit_downloader.download import download_data


def main():
    config = Config.load_recursive()
    download_data(config)


if __name__ == "__main__":
    main()



import datetime
from enum import Enum
from pathlib import Path
from typing import Final, Optional, List

from pyappconf import BaseConfig, AppConfig, ConfigFormats
from pydantic import BaseModel, Field

APP_NAME: Final = "fitbit-downloader"


class OAuthConfig(BaseModel):
    access_token: str
    refresh_token: str
    expires_at: float

    @property
    def expires_at_time(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.expires_at)


class Dataset(str, Enum):
    STEPS = "steps"
    HEART = "heart"
    ELEVATION = "elevation"
    DISTANCE = "distance"
    FLOORS = "floors"
    SLEEP = "sleep"
    ACTIVITIES = "activities"


class DownloadConfig(BaseModel):
    datasets: List[Dataset] = Field(
        default_factory=lambda: [
            Dataset.STEPS,
            Dataset.HEART,
            Dataset.ELEVATION,
            Dataset.DISTANCE,
            Dataset.FLOORS,
            Dataset.SLEEP,
            Dataset.ACTIVITIES,
        ]
    )
    out_folder: Path = Path(__file__).parent.parent / "Data"


class Config(BaseConfig):
    client_id: str
    client_secret: str
    oauth_config: Optional[OAuthConfig]
    download: DownloadConfig = DownloadConfig()

    _settings = AppConfig(
        app_name=APP_NAME,
        default_format=ConfigFormats.TOML,
        config_name="fitbit-downloader",
    )

    class Config:
        env_file = ".env"
        env_prefix = "FITBIT_DOWNLOADER_"


if __name__ == "__main__":
    print(Config.load_recursive())



# generated by datamodel-codegen:
#   filename:  activity.json
#   timestamp: 2021-10-31T18:17:21+00:00

from __future__ import annotations

from typing import List

from pydantic import BaseModel


class Activity(BaseModel):
    activityId: int
    activityParentId: int
    activityParentName: str
    calories: int
    description: str
    duration: int
    hasActiveZoneMinutes: bool
    hasStartTime: bool
    isFavorite: bool
    lastModified: str
    logId: int
    name: str
    startDate: str
    startTime: str
    steps: int


class Goals(BaseModel):
    activeMinutes: int
    caloriesOut: int
    distance: int
    floors: int
    steps: int


class Distance(BaseModel):
    activity: str
    distance: float


class HeartRateZone(BaseModel):
    caloriesOut: float
    max: int
    min: int
    minutes: int
    name: str


class Summary(BaseModel):
    activeScore: int
    activityCalories: int
    caloriesBMR: int
    caloriesOut: int
    distances: List[Distance]
    elevation: int
    fairlyActiveMinutes: int
    floors: int
    heartRateZones: List[HeartRateZone]
    lightlyActiveMinutes: int
    marginalCalories: int
    restingHeartRate: int
    sedentaryMinutes: int
    steps: int
    veryActiveMinutes: int


class ActivityResponse(BaseModel):
    activities: List[Activity]
    goals: Goals
    summary: Summary



# generated by datamodel-codegen:
#   filename:  sleep.json
#   timestamp: 2021-10-31T18:17:20+00:00

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel


class MinuteDatum(BaseModel):
    dateTime: str
    value: str


class SleepItem(BaseModel):
    awakeCount: int
    awakeDuration: int
    awakeningsCount: int
    dateOfSleep: str
    duration: int
    efficiency: int
    endTime: str
    isMainSleep: bool
    logId: int
    minuteData: List[MinuteDatum]
    minutesAfterWakeup: int
    minutesAsleep: int
    minutesAwake: int
    minutesToFallAsleep: int
    restlessCount: int
    restlessDuration: int
    startTime: str
    timeInBed: int


class Stages(BaseModel):
    deep: int
    light: int
    rem: int
    wake: int


class Summary(BaseModel):
    # TODO: Set this to optional dynamically
    stages: Optional[Stages]
    totalMinutesAsleep: int
    totalSleepRecords: int
    totalTimeInBed: int


class SleepResponse(BaseModel):
    sleep: List[SleepItem]
    summary: Summary



# This is the main settings file for package setup and PyPi deployment.
# Sphinx configuration is in the docsrc folder

# Main package name
PACKAGE_NAME = "fitbit-downloader"

# Directory name of package
PACKAGE_DIRECTORY = "fitbit_downloader"

# Name of Repo
REPO_NAME = "fitbit-downloader"

# Github username of the user which owns the repo
REPO_USERNAME = "nickderobertis"

# List of maintainers of package, by default the same user which owns the repo
# Pull requests raised by these maintainers without the "no auto merge" label will be automatically merged
REPO_MAINTAINERS = [
    REPO_USERNAME,
]

# Package version in the format (major, minor, release)
PACKAGE_VERSION_TUPLE = (0, 1, 0)

# Short description of the package
PACKAGE_SHORT_DESCRIPTION = "A CLI and Python tool for downloading Fitbit data"

# Long description of the package for PyPI
# Set to 'auto' to use README.md as the PyPI description
# Any other string will be used directly as the PyPI description
PACKAGE_DESCRIPTION = 'auto'

# Author
PACKAGE_AUTHOR = "Nick DeRobertis"

# Author email
PACKAGE_AUTHOR_EMAIL = "whoopnip@gmail.com"

# Name of license for package
PACKAGE_LICENSE = 'MIT'

# Classifications for the package, see common settings below
PACKAGE_CLASSIFIERS = [
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 3 - Alpha',

    # Indicate who your project is intended for
    'Intended Audience :: Developers',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'
]

# Add any third party packages you use in requirements here
PACKAGE_INSTALL_REQUIRES = [
    # Include the names of the packages and any required versions in as strings
    # e.g.
    # 'package',
    # 'otherpackage>=1,<2'
]

# Add any third party packages you use in requirements for optional features of your package here
# Keys should be name of the optional feature and values are lists of required packages
# E.g. {'feature1': ['pandas', 'numpy'], 'feature2': ['matplotlib']}
OPTIONAL_PACKAGE_INSTALL_REQUIRES = {

}

# Packages added to Binder environment so that examples can be executed in Binder
# By default, takes this package (PACKAGE_NAME)
# everything the package requires (PACKAGE_INSTALL_REQUIRES) and everything
# that the package optionally requires (OPTIONAL_PACKAGE_INSTALL_REQUIRES) and adds them all to one list
# If a custom list is passed, it must include all the requirements for the Binder environment
BINDER_ENVIRONMENT_REQUIRES = list(
    set(
        PACKAGE_INSTALL_REQUIRES + [PACKAGE_NAME] +
        [package for package_list in OPTIONAL_PACKAGE_INSTALL_REQUIRES.values() for package in package_list]
    )
)


# Sphinx executes all the import statements as it generates the documentation. To avoid having to install all
# the necessary packages, third-party packages can be passed to mock imports to just skip the import.
# By default, everything in PACKAGE_INSTALL_REQUIRES will be passed as mock imports, along with anything here.
# This variable is useful if a package includes multiple packages which need to be ignored.
DOCS_OTHER_MOCK_IMPORTS = [
    # Include the names of the packages as they would be imported, e.g.
    # 'package',
]

# Add any Python scripts which should be exposed to the command line in the format:
# CONSOLE_SCRIPTS = ['funniest-joke=funniest.command_line:main']
CONSOLE_SCRIPTS = [],

# Add any arbitrary scripts to be exposed to the command line in the format:
# SCRIPTS = ['bin/funniest-joke']
SCRIPTS = []

# Optional Google Analytics tracking ID for documentation
# Go to https://analytics.google.com/ and set it up for your documentation URL
# Set to None or empty string to not use this
GOOGLE_ANALYTICS_TRACKING_ID = "G-1FVMQJ01HT"

PACKAGE_URLS = {
    'Code': f'https://github.com/{REPO_USERNAME}/{REPO_NAME}',
    'Documentation': f'https://{REPO_USERNAME}.github.io/{REPO_NAME}'
}

# Url of logo
PACKAGE_LOGO_URL = ""

# Does not affect anything about the current package. Simply used for tracking when this repo was created off
# of the quickstart template, so it is easier to bring over new changes to the template.
_TEMPLATE_VERSION_TUPLE = (0, 9, 2)

if __name__ == '__main__':
    # Store config as environment variables
    env_vars = dict(locals())
    # Imports after getting locals so that locals are only environment variables
    import shlex
    for name, value in env_vars.items():
        quoted_value = shlex.quote(str(value))
        print(f'export {name}={quoted_value};')



import datetime
from typing import Any

from fitbit_downloader.config import Config, OAuthConfig
from fitbit_downloader.gather_keys_oauth2 import OAuth2Server
from fitbit_downloader.logger import logger
from fitbit_downloader.models.refreshtokenresponse import RefreshTokenResponse


def initial_client_authorization(config: Config):
    server = OAuth2Server(config.client_id, config.client_secret)
    server.browser_authorize()

    profile = server.fitbit.user_profile_get()
    logger.info(
        "You are authorized to access data for the user: {}".format(
            profile["user"]["fullName"]
        )
    )

    set_authorization(config, server.fitbit.client.session.token)


def set_authorization(config: Config, raw_token_data: dict[str, Any]):
    token_data = RefreshTokenResponse(**raw_token_data)
    _update_config_with_token_data(config, token_data)
    logger.info(f"Token refreshed. New expiry: {config.oauth_config.expires_at_time}")  # type: ignore
    config.save()


def _update_config_with_token_data(config: Config, token_data: RefreshTokenResponse):
    oauth_config = OAuthConfig(
        access_token=token_data.access_token,
        refresh_token=token_data.refresh_token,
        expires_at=token_data.expires_at,
    )
    config.oauth_config = oauth_config



import datetime
import json
from pathlib import Path

import fitbit

from fitbit_downloader.client import get_client
from fitbit_downloader.config import Config
from fitbit_downloader.logger import logger
from tests.config import INTRADAY_RESPONSES_FOLDER, RESPONSES_FOLDER


def download_sample_responses(client: fitbit.Fitbit):
    for activity_type in ("steps", "heart", "elevation", "distance", "floors"):
        activity_str = f"activities/{activity_type}"
        logger.info(f"Downloading {activity_str}")
        out_path = INTRADAY_RESPONSES_FOLDER / f"{activity_type}.json"
        data = client.intraday_time_series(activity_str)
        _write_json(data, out_path)

    logger.info("Downloading sleep")
    sleep_data = client.get_sleep(datetime.date.today())
    sleep_out_path = RESPONSES_FOLDER / "sleep.json"
    _write_json(sleep_data, sleep_out_path)

    logger.info("Downloading activities")
    activity_data = client.activities()  # type: ignore
    activity_out_path = RESPONSES_FOLDER / "activity.json"
    _write_json(activity_data, activity_out_path)


def _write_json(data: dict, path: Path):
    path.write_text(json.dumps(data, indent=2))


if __name__ == "__main__":
    config = Config.load_recursive()
    client = get_client(config)
    download_sample_responses(client)



from pathlib import Path
from typing import Type

from pydantic import BaseModel

from fitbit_downloader.models.activityresponse import ActivityResponse
from fitbit_downloader.models.distanceresponse import DistanceResponse
from fitbit_downloader.models.elevationresponse import ElevationResponse
from fitbit_downloader.models.floorsresponse import FloorsResponse
from fitbit_downloader.models.heartresponse import HeartResponse
from fitbit_downloader.models.refreshtokenresponse import RefreshTokenResponse
from fitbit_downloader.models.sleepresponse import SleepResponse
from fitbit_downloader.models.stepsresponse import StepsResponse
from tests.config import INTRADAY_RESPONSES_FOLDER, RESPONSES_FOLDER


def _model_construction_test(path: Path, model_cls: Type[BaseModel]):
    model_cls.parse_file(path)


def test_elevation_response():
    json_path = INTRADAY_RESPONSES_FOLDER / "elevation.json"
    _model_construction_test(json_path, ElevationResponse)


def test_heart_response():
    json_path = INTRADAY_RESPONSES_FOLDER / "heart.json"
    _model_construction_test(json_path, HeartResponse)


def test_steps_response():
    json_path = INTRADAY_RESPONSES_FOLDER / "steps.json"
    _model_construction_test(json_path, StepsResponse)


def test_distance_response():
    json_path = INTRADAY_RESPONSES_FOLDER / "distance.json"
    _model_construction_test(json_path, DistanceResponse)


def test_floors_response():
    json_path = INTRADAY_RESPONSES_FOLDER / "floors.json"
    _model_construction_test(json_path, FloorsResponse)


def test_sleep_response():
    json_path = RESPONSES_FOLDER / "sleep.json"
    _model_construction_test(json_path, SleepResponse)


def test_activity_response():
    json_path = RESPONSES_FOLDER / "activity.json"
    _model_construction_test(json_path, ActivityResponse)


def test_refresh_token_response():
    json_path = RESPONSES_FOLDER / "refresh-token.json"
    _model_construction_test(json_path, RefreshTokenResponse)
